package kcommon;
#
# CONSTANTS, PARAMETERS and COMMON ROUTINES
# for the various kerberos utilities and tools:
#	kstats		- statistics reporting tools
#	kprinc		- add/delete/modify kerberos principals
#	kcroninit	- initialize keytab and cron principals
# etc.

use English;

use Exporter;

@ISA = qw( Exporter );
@EXPORT = qw(
	$DEFAULT_REALM
	$KRB5CONF
	%KDC_NODE
	$SUCCESS
	$FAILURE
	$TRUE
	$FALSE
	$DEBUG
	$CURRENT_VERBOSITY
	$ERRWIDTH
	$ERRMSGS
	parsecmd
	abort
	dbgprint
	readword
	readwithDefault
	readpw
	parse_principal
	);

use vars qw(
	@ISA
	@EXPORT
	$DEFAULT_REALM
	$KRB5CONF
	%KDC_NODE
	$SUCCESS
	$FAILURE
	$TRUE
	$FALSE
	$DEBUG
	$CURRENT_VERBOSITY
	$ERRWIDTH
	$ERRMSGS
	);

BEGIN {
    $KRB5CONF = "/etc/krb5.conf";					# constant
    $SUCCESS = 1;							# constant
    $FAILURE = 0;							# constant
    $TRUE    = 1;							# constant
    $FALSE   = 0;							# constant
    $DEBUG   = 0;							# no debugging unless set by commandline -d
    $CURRENT_VERBOSITY = 0;						# verbosity level (possibly reset based on cmdline -vvvv)
    $ERRWIDTH = 255;							# width for wrapping error messages
    $ERRMSGS  = "";							# running list of error messages
    $DEFAULT_REALM = "FNAL.GOV";					# constant
    my @defrealm = grep /^\s*default_realm\s*=/, qx( cat $KRB5CONF );	# Look for the realm default realm!
    if ( $#defrealm >= 0
	&& $defrealm[0] =~ /^\s*default_realm\s*=\s*(\S+)\s*/ ) {
      $DEFAULT_REALM = $1;
    }
}
################################################################################

#
# Process the command line arguments and options, returning the arguments
# and options found.
#
# Input : $control -- Control string.  Use getopts(3) format.
#         \@argv -- input arguments
#
# Output: $flags -- Reference to a hash containing summary
#                   of command line options.
#         $args  -- Reference to an array containing the arguments
#                   from the command line.
#
# Return: $status -- Always true.
#

sub parsecmd {

#
#  Passed arguments.
#

    my $control = shift() . " ";		#  Option parsing control string.
                    				#  The extra space is for the
                    				#  convenience of the argument checker.

    my $inargs = shift();			#  Array ref for @ARGV
    my $outflags = shift();			#  Hash ref for command line options.
    my $outargs = shift();			#  Array ref for command line arguments.

    #
    #  Private variables.
    #

    my $fc;					#  First character (option letter).
    my $rest;					#  Rest of option after first character.
    my $x;					#  Index into control string.

    #
    #  Check passed arguments.
    #

    if (       not defined( $outargs )
        or not ref( $outargs )
        or ( ref( $outargs ) ne "ARRAY" )
    ) {
        die( "Invalid arguments!, stopped" );
    }

    if (       not defined( $outflags )
        or not ref( $outflags )
        or ( ref( $outflags ) ne "HASH" )
    ) {
        die( "Invalid arguments!, stopped" );
    }

    #
    #  Loop over all the arguments.
    #

    while ( @{$inargs} ) {

	#
	#  Get the next argument.
	#

        $_ = shift( @{$inargs} );

	#
	#  Is it an option?
	#

	if ( not /^-(.)(.*)/ ) {

	#
	#  Not an option, we have an argument.  Save away,
	#  so we can return it to our caller.
	#

            push( @{$outargs}, $_ );

	#
	#  Move on to next argument.
	#

            next;

	}

	#
	#    Ok, we had an option.  Pull it out of the argument and
	#    keep the rest for option clustering and possibly for use
	#    as an argument to the option.
	#

        ( $fc, $rest ) = ( $1, $2 );

	#
	#  Look up the option letter in the passed control string.
	#

        $x = index( $control, $fc );

	#
	#  Did we match an option in the control string?
	#

        if ( -1 == $x ) {

	    #
	    #  Nope, crash and burn.
	    #

            die( "Unknown option: -$fc, stopped" );

        } else {

	    #
	    #  Ok, we have a legal option.  Does it
	    #  require an argument?
	    #

	    #
	    #  N.B.:  This is where we needed the extra
	    #         space added to the control string
	    #         ( so we can go one char past nominal end).
	    #

            if ( substr( $control, $x + 1, 1 ) eq ":" ) {

		#
		#  Ok, we need an argument for the option.
		#  Use what was included with the option,
		#  if anything, otherwise use the next
		#  argument.
		#

                if (        defined( $rest )
                    and ( $rest ne "" )
                ) {
                    $_ = $rest;
                } else {
                    $_ = shift( @{$inargs} );
                }

                if ( not defined( $_ ) ) {
		    die( "Option -$fc requires an argument!, stopped" );
                }

		#
		#  We got the option's required argument.
		#  Put it into the flags hash.
		#

                $outflags->{"opt_$fc"} = $_;

            } else {

		#
		#  We got a legal option which needs no
		#  argument.  Update the count for the option
		#  in the flags hash.
		#

		#
		#  Handle the -? option in a special way,
		#  since '?' is invalid in variable names.
		#
                
                if ( $fc eq "?" ) {
                    $fc = "help";
                }

                if ( not exists( $outflags->{"opt_$fc"} ) ) {
                    $outflags->{"opt_$fc"} = 1;
                } else {
                    $outflags->{"opt_$fc"}++;
                }

		#
		# Allow option clustering.
		#

                if (        defined( $rest )
                    and ( $rest ne "" )
                ) {
                    unshift( @{$inargs}, "-" . $rest );
                }

            }

        }

    }

#
#  Return to caller with success status.
#

    return( $SUCCESS );

}
################################################################################
#
sub USAGE {
    my ($PROG) = fileparse($PROGRAM_NAME);
    my ($thing, $dsc, $valid_options, $report_type, $option_type);

    if ( ! $CMDRPT ) {								# haven't selected a specific type of report yet
	$valid_options = $ALL_VALID_OPTS;
	$report_type = "report-type";
	$option_type = "All valid options";
    } else {									# here we've asked for specific help on one report
	$valid_options = $VALID_OPTS;
	$report_type = $CMDRPT;
	$option_type = "Valid options";
    }
    my (@options) = ( $valid_options =~ m/([a-zA-Z]{1}:|[a-zA-Z]{1})/g );

    print STDOUT <<EOUSAGE;

    USAGE:

	$PROG $report_type [options] infilelist
EOUSAGE

    if ( not $CMDRPT ) {
    print STDOUT ("\n    Report types:\n\n");
	foreach $thing ( sort (keys %RPT_TYPE) ) {
	    $dsc = $RPT_TYPE{"$thing"};
	    print STDOUT ("\t$dsc\n");
	}
    }

    print STDOUT ("\n    $option_type:\n\n");
    foreach $thing ( sort (@options) ) {
	$dsc = $OPT{"$thing"};
	if ( $thing =~ /:$/ ) {							# if options has :, it takes a value
	    print STDOUT ("\t-$thing value\t$dsc\n");
	} else {
	    print STDOUT ("\t-$thing\t\t$dsc\n");
	}
    }
    print STDOUT ("\n    (Not all options are valid for all commands).\n") if ( not $CMDRPT );
    return $SUCCESS;
}
################################################################################
#
sub parse_principal {
    my ($principal) = @_;							# primary/instance@realm

    my ( $primary, $instance, $realm );
    my $len;
    my ( $iSlash,   $iAt);
    my ( $hasSlash, $hasAt );

    $len = length($principal);							# length of the principal passed
    $iSlash = index($principal, "/");						# first occurance of "/"
    $iAt    = rindex($principal, "@");						# last occurance of "@"

    $hasSlash = ($iSlash >= 0);							# boolean flag
    $hasAt    = ($iAt    >= 0);							# boolean flag

    if ( $hasSlash && $hasAt ) {						# all pieces are fully specified ("lauri/cron/node@REALM")
	$primary  = substr($principal, 0, $iSlash);				# primary up to "/" (lauri)
	$instance = substr($principal, $iSlash+1, ($iAt - $iSlash - 1));	# instance from first "/" to "@" (cron/node)
	$realm    = substr($principal, $iAt+1, $len);				# realm from "@" to the end (REALM)
    } elsif ( $hasSlash ) {							# only the "/" was specified ("lauri/cron/node")
	$primary  = substr($principal, 0, $iSlash);				# primary up to "/" (lauri)
	$instance = substr($principal, $iSlash+1, $len);			# instance to the end ("cron/node")
	$realm    = $DEFAULT_REALM;						# default realm
    } elsif ( $hasAt ) {							# no instance specified ("lauri@REALM")
	$primary  = substr($principal, 0, $iAt);				# primary up to "@" (lauri)
	$instance = "";								# no instance
	$realm    = substr($principal, $iAt+1, $len);				# realm up to the end (REALM)
    } else {									# only the primary was specified (lauri)
	$primary  = $principal;							# primary is the principal (lauri)
	$instance = "";								# no instance
	$realm    = $DEFAULT_REALM;						# default realm
    }

    dbgprint("parse_principal >$principal< yields primary = >$primary<  instance = >$instance<  realm = >$realm<\n");
    return ( $primary, $instance, $realm );					# send it back
}
################################################################################

sub abort {
    my ($message) = @_;
    die("$message");
}
################################################################################
#
# dbgprint($message)
#
# Print a message if $DEBUG is set.
sub dbgprint {
    my ($message) = @_;
    print STDERR $message if ($DEBUG);
}
#
################################################################################
#
# read a line of input, trim leading and trailing whitespace, ship it back
#
sub readword {
    my ($prompt) = @_;
    my ($line);
    print STDOUT ("$prompt");			# prompt
    $line = <STDIN>;				# read input
    chomp($line);				# strip trailing linefeed
    $line =~ s/^\s+//;				# strip leading whitespace
    $line =~ s/\s+$//;				# strip trailing whitespace
    return $line;				# ship it back
}
################################################################################
#
# read a line of input using readword, but have a default value if the
# user enters nothing
#
sub readwithDefault {
    my ($prompt, $default) = @_;
    my $entry = "";
    my $defaultMessage = "";
    if ( defined($default) ) {
	$defaultMessage = " (default = $default)";
    }

    $entry = readword("$prompt$defaultMessage: ");
    $entry = $default if ( !$entry );
    return ($entry);
}
################################################################################
#
# read a password string.
# Input:
#	prompt string
# Return:
#	password entered
# ByProducts:
#	none
#
sub readpw {
    my ($prompt) = @_;						# prompt is passed in
    dbgprint("into readps for prompt = >$prompt<\n");

    print STDOUT $prompt;					# print the prompt string
    system("stty -echo");					# turn off echo
    my $passwd = <STDIN>;					# read the entered string
    chomp($passwd);						# remove trailing \n if found
    system("stty echo");					# turn echo back on
    print STDOUT ("\n");					# move to next line

    return($passwd);						# return the string
}

################################################################################

return( $SUCCESS );
